@@ -15,7 +15,11 @@ from .plotting.animate import animate_poloidal, animate_pcolormesh, animate_line
 from .plotting import plotfuncs
 from .plotting.utils import _create_norm
 from .region import _from_region
-from .utils import _update_metadata_increased_resolution, _get_bounding_surfaces
+from .utils import (
+    _add_cartesian_coordinates,
+    _update_metadata_increased_resolution,
+    _get_bounding_surfaces,
+)
 
 
 @register_dataarray_accessor("bout")
@@ -118,7 +122,7 @@ class BoutDataArrayAccessor:
 
         data_shifted_fft = data_fft * np.exp(phase.data)
 
-        data_shifted = fft.irfft(data_shifted_fft, n=nz)
+        data_shifted = fft.irfft(data_shifted_fft, n=nz, axis=axis)
 
         # Return a DataArray with the same attributes as self, but values from
         # data_shifted
@@ -189,7 +193,7 @@ class BoutDataArrayAccessor:
         return result
 
     @property
-    def regions(self):
+    def _regions(self):
         if "regions" not in self.data.attrs:
             raise ValueError(
                 "Called a method requiring regions, but these have not been created. "
@@ -279,7 +283,7 @@ class BoutDataArrayAccessor:
                 self.interpolate_parallel(
                     region, n=n, toroidal_points=toroidal_points, method=method
                 ).bout.to_dataset()
-                for region in self.data.regions
+                for region in self._regions
             ]
 
             # 'region' is not the same for all parts, and should not exist in the result,
@@ -301,7 +305,7 @@ class BoutDataArrayAccessor:
 
         # Select a particular 'region' and interpolate to higher parallel resolution
         da = self.data
-        region = da.regions[region]
+        region = da.bout._regions[region]
         tcoord = da.metadata["bout_tdim"]
         xcoord = da.metadata["bout_xdim"]
         ycoord = da.metadata["bout_ydim"]
@@ -381,6 +385,17 @@ class BoutDataArrayAccessor:
 
         return da
 
+    def add_cartesian_coordinates(self):
+        """
+        Add Cartesian (X,Y,Z) coordinates.
+
+        Returns
+        -------
+        DataArray with new coordinates added, which are named 'X_cartesian',
+        'Y_cartesian', and 'Z_cartesian'
+        """
+        return _add_cartesian_coordinates(self.data)
+
     def remove_yboundaries(self, return_dataset=False, remove_extra_upper=False):
         """
         Remove y-boundary points, if present, from the DataArray
@@ -409,9 +424,9 @@ class BoutDataArrayAccessor:
 
         ycoord = self.data.metadata["bout_ydim"]
         parts = []
-        for region in self.data.regions:
+        for region in self._regions:
             part = self.data.bout.from_region(region, with_guards=0)
-            part_region = list(part.regions.values())[0]
+            part_region = list(part.bout._regions.values())[0]
             if part_region.connection_lower_y is None:
                 part = part.isel({ycoord: slice(myg, None)})
             if part_region.connection_upper_y is None:
@@ -525,7 +540,7 @@ class BoutDataArrayAccessor:
         if region is None:
             # Call the single-region version of this method for each region, and combine
             # the results together
-            parts = [self.ddy(r).to_dataset() for r in self.data.regions]
+            parts = [self.ddy(r).to_dataset() for r in self._regions]
 
             # 'region' is not the same for all parts, and should not exist in the
             # result, so delete before merging
@@ -537,7 +552,7 @@ class BoutDataArrayAccessor:
             result = xr.combine_by_coords(parts)[f"d({name})/dy"]
 
             # regions get mixed up during the split and combine_by_coords, so reset them
-            result.attrs["regions"] = self.data.regions
+            result.attrs["regions"] = self._regions
 
             return result
 
@@ -568,7 +583,7 @@ class BoutDataArrayAccessor:
         result = (da.shift({ycoord: -1}) - da.shift({ycoord: 1})) / (2.0 * dy)
 
         # Remove any y-guard cells
-        region_object = da.regions[region]
+        region_object = da.bout._regions[region]
         if region_object.connection_lower_y is None:
             ylower = None
         else:
@@ -583,6 +598,9 @@ class BoutDataArrayAccessor:
             # Want output in non-aligned coordinates
             result = result.bout.from_field_aligned()
 
+        if "regions" in result.attrs:
+            del result.attrs["regions"]
+
         result.name = f"d({da.name})/dy"
         if "standard_name" in result.attrs:
             result.attrs["standard_name"] = f"d({result.attrs['standard_name']})/dy"
@@ -998,15 +1016,64 @@ class BoutDataArrayAccessor:
 
         return result
 
+    def interpolate_to_cartesian(self, *args, **kwargs):
+        """
+        Interpolate the DataArray to a regular Cartesian grid.
+
+        This method is intended to be used to produce data for visualisation, which
+        normally does not require double-precision values, so by default the data is
+        converted to `np.float32`. Pass `use_float32=False` to retain the original
+        precision.
+
+        Parameters
+        ----------
+        nX : int (default 300)
+            Number of grid points in the X direction
+        nY : int (default 300)
+            Number of grid points in the Y direction
+        nZ : int (default 100)
+            Number of grid points in the Z direction
+        use_float32 : bool (default True)
+            Downgrade precision to `np.float32`?
+        fill_value : float (default np.nan)
+            Value to use for points outside the interpolation domain (passed to
+            `scipy.RegularGridInterpolator`)
+
+        See Also
+        --------
+        BoutDataset.interpolate_to_cartesian
+        """
+        da = self.data
+        name = da.name
+        ds = da.to_dataset()
+        # Dataset needs geometry and metadata attributes, but these are not copied from
+        # the DataArray by default
+        ds.attrs["geometry"] = da.geometry
+        ds.attrs["metadata"] = da.metadata
+        return ds.bout.interpolate_to_cartesian(*args, **kwargs)[name]
+
     # BOUT-specific plotting functionality: methods that plot on a poloidal (R-Z) plane
     def contour(self, ax=None, **kwargs):
+        """
+        Contour-plot a radial-poloidal slice on the R-Z plane
+        """
         return plotfuncs.plot2d_wrapper(self.data, xr.plot.contour, ax=ax, **kwargs)
 
     def contourf(self, ax=None, **kwargs):
+        """
+        Filled-contour-plot a radial-poloidal slice on the R-Z plane
+        """
         return plotfuncs.plot2d_wrapper(self.data, xr.plot.contourf, ax=ax, **kwargs)
 
     def pcolormesh(self, ax=None, **kwargs):
+        """
+        Colour-plot a radial-poloidal slice on the R-Z plane
+        """
         return plotfuncs.plot2d_wrapper(self.data, xr.plot.pcolormesh, ax=ax, **kwargs)
 
-    def regions(self, ax=None, **kwargs):
-        return plotfuncs.regions(self.data, ax=ax, **kwargs)
+    def plot_regions(self, ax=None, **kwargs):
+        """
+        Plot the regions into which xBOUT splits radial-poloidal arrays to handle
+        tokamak topology.
+        """
+        return plotfuncs.plot_regions(self.data, ax=ax, **kwargs)
