@@ -4,7 +4,7 @@ from textwrap import dedent
 import xarray as xr
 import numpy as np
 
-from .region import Region, _create_regions_toroidal
+from .region import Region, _create_regions_toroidal, _create_single_region
 from .utils import (
     _add_attrs_to_var,
     _set_attrs_on_all_vars,
@@ -56,8 +56,6 @@ def apply_geometry(ds, geometry_name, *, coordinates=None, grid=None):
         ds = _set_attrs_on_all_vars(ds, "geometry", "")
         updated_ds = ds
     else:
-        ds = _set_attrs_on_all_vars(ds, "geometry", geometry_name)
-
         try:
             add_geometry_coords = REGISTERED_GEOMETRIES[geometry_name]
         except KeyError:
@@ -81,6 +79,11 @@ def apply_geometry(ds, geometry_name, *, coordinates=None, grid=None):
         else:
             updated_ds = add_geometry_coords(ds)
 
+        # Set "geometry" attribute after adding coordinates, so that functions in
+        # `REGISTERED_GEOMETRIES` can check if ds.attrs["geometry"] is already defined
+        # to see if they are being applied for the first time or re-applied.
+        updated_ds = _set_attrs_on_all_vars(updated_ds, "geometry", geometry_name)
+
     del ds
 
     # Set dimension names if they were not set by add_geometry_coords(). Dimensions
@@ -183,7 +186,10 @@ def apply_geometry(ds, geometry_name, *, coordinates=None, grid=None):
 
         # In BOUT++ v5, dz is either a Field2D or Field3D.
         # We can use it as a 1D coordinate if it's a Field3D, _or_ if nz == 1
-        bout_v5 = updated_ds.metadata["BOUT_VERSION"] >= 5.0
+        bout_version = updated_ds.metadata.get("BOUT_VERSION", 4.3)
+        bout_v5 = bout_version > 5.0 or (
+            bout_version == 5.0 and updated_ds["dz"].ndim >= 2
+        )
         use_metric_3d = updated_ds.metadata.get("use_metric_3d", False)
         can_use_1d_z_coord = (nz == 1) or use_metric_3d
 
@@ -195,14 +201,20 @@ def apply_geometry(ds, geometry_name, *, coordinates=None, grid=None):
                     raise ValueError(
                         f"Spacing is not constant. Cannot create z coordinate"
                     )
-                dz = updated_ds["dz"][0, 0]
+
+                dz = updated_ds["dz"].min()
             else:
                 dz = updated_ds["dz"]
 
             z0 = 2 * np.pi * updated_ds.metadata["ZMIN"]
             z1 = z0 + nz * dz
-            if not np.isclose(
-                z1, 2.0 * np.pi * updated_ds.metadata["ZMAX"], rtol=1.0e-15, atol=0.0
+            if not np.all(
+                np.isclose(
+                    z1,
+                    2.0 * np.pi * updated_ds.metadata["ZMAX"],
+                    rtol=1.0e-15,
+                    atol=0.0,
+                )
             ):
                 warn(
                     f"Size of toroidal domain as calculated from nz*dz ({str(z1 - z0)}"
@@ -270,7 +282,10 @@ def _set_default_toroidal_coordinates(coordinates, ds):
         coordinates = {}
 
     # Replace any values that have not been passed in with defaults
-    coordinates["t"] = coordinates.get("t", ds.metadata["bout_tdim"])
+    if ds.metadata["is_restart"] == 0:
+        # Don't need "t" coordinate for restart files which have no time dimension, and
+        # adding it breaks the check for reloading in open_boutdataset
+        coordinates["t"] = coordinates.get("t", ds.metadata["bout_tdim"])
 
     default_x = (
         ds.metadata["bout_xdim"] if ds.metadata["bout_xdim"] != "x" else "psi_poloidal"
@@ -286,31 +301,9 @@ def _set_default_toroidal_coordinates(coordinates, ds):
     return coordinates
 
 
-@register_geometry("toroidal")
-def add_toroidal_geometry_coords(ds, *, coordinates=None, grid=None):
-
-    coordinates = _set_default_toroidal_coordinates(coordinates, ds)
-
-    if set(coordinates.values()).issubset(set(ds.coords).union(ds.dims)):
-        # Loading a Dataset which already had the coordinates created for it
-        ds = _create_regions_toroidal(ds)
-        return ds
-
-    # Check whether coordinates names conflict with variables in ds
-    bad_names = [
-        name for name in coordinates.values() if name in ds and name not in ds.coords
-    ]
-    if bad_names:
-        raise ValueError(
-            "Coordinate names {} clash with variables in the dataset. "
-            "Register a different geometry to provide alternative names. "
-            "It may be useful to use the 'coordinates' argument to "
-            "add_toroidal_geometry_coords() for this.".format(bad_names)
-        )
-
+def _add_vars_from_grid(ds, grid, variables, *, optional_variables=None):
     # Get extra geometry information from grid file if it's not in the dump files
-    needed_variables = ["psixy", "Rxy", "Zxy"]
-    for v in needed_variables:
+    for v in variables:
         if v not in ds:
             if grid is None:
                 raise ValueError(
@@ -328,6 +321,64 @@ def add_toroidal_geometry_coords(ds, *, coordinates=None, grid=None):
 
             _add_attrs_to_var(ds, v)
 
+    if optional_variables is not None:
+        for v in optional_variables:
+            if v not in ds:
+                if grid is None:
+                    continue
+                if v in grid:
+                    # ds[v] = grid[v]
+                    # Work around issue where xarray drops attributes on
+                    # coordinates when a new DataArray is assigned to the
+                    # Dataset, see https://github.com/pydata/xarray/issues/4415
+                    # https://github.com/pydata/xarray/issues/4393
+                    # This way adds as a 'Variable' instead of as a 'DataArray'
+                    ds[v] = (grid[v].dims, grid[v].values)
+
+                    _add_attrs_to_var(ds, v)
+
+    return ds
+
+
+@register_geometry("toroidal")
+def add_toroidal_geometry_coords(ds, *, coordinates=None, grid=None):
+
+    coordinates = _set_default_toroidal_coordinates(coordinates, ds)
+
+    if ds.attrs.get("geometry", None) == "toroidal":
+        # Loading a Dataset which already had the coordinates created for it
+        ds = _create_regions_toroidal(ds)
+        return ds
+
+    # Check whether coordinates names conflict with variables in ds
+    bad_names = [
+        name for name in coordinates.values() if name in ds and name not in ds.coords
+    ]
+    if bad_names:
+        raise ValueError(
+            "Coordinate names {} clash with variables in the dataset. "
+            "Register a different geometry to provide alternative names. "
+            "It may be useful to use the 'coordinates' argument to "
+            "add_toroidal_geometry_coords() for this.".format(bad_names)
+        )
+
+    # Get extra geometry information from grid file if it's not in the dump files
+    ds = _add_vars_from_grid(
+        ds,
+        grid,
+        ["psixy", "Rxy", "Zxy"],
+        optional_variables=[
+            "Bpxy",
+            "Brxy",
+            "Bzxy",
+            "poloidal_distance",
+            "poloidal_distance_ylow",
+            "total_poloidal_distance",
+            "zShift",
+            "zShift_ylow",
+        ],
+    )
+
     if "t" in ds.dims:
         # Rename 't' if user requested it
         ds = ds.rename(t=coordinates["t"])
@@ -341,7 +392,8 @@ def add_toroidal_geometry_coords(ds, *, coordinates=None, grid=None):
     ds[coordinates["x"]].attrs["units"] = "Wb"
 
     # Record which dimensions 't', 'x', and 'y' were renamed to.
-    ds.metadata["bout_tdim"] = coordinates["t"]
+    if ds.metadata["is_restart"] == 0:
+        ds.metadata["bout_tdim"] = coordinates["t"]
     # x dimension not renamed, so this is still 'x'
     ds.metadata["bout_xdim"] = "x"
     ds.metadata["bout_ydim"] = coordinates["y"]
@@ -363,8 +415,23 @@ def add_toroidal_geometry_coords(ds, *, coordinates=None, grid=None):
     else:
         ds = ds.set_coords(("Rxy", "Zxy"))
 
+    # Rename zShift_ylow if it was added from grid file, to be consistent with name if
+    # it was added from dump file
+    if "zShift_CELL_YLOW" in ds and "zShift_ylow" in ds:
+        # Remove redundant copy
+        del ds["zShift_ylow"]
+    elif "zShift_ylow" in ds:
+        ds = ds.rename(zShift_ylow="zShift_CELL_YLOW")
+
+    if "poloidal_distance" in ds:
+        ds = ds.set_coords(
+            ["poloidal_distance", "poloidal_distance_ylow", "total_poloidal_distance"]
+        )
+
     # Add zShift as a coordinate, so that it gets interpolated along with a variable
     ds = _set_as_coord(ds, "zShift")
+    if "zShift_CELL_YLOW" in ds:
+        ds = _set_as_coord(ds, "zShift_CELL_YLOW")
 
     ds = _create_regions_toroidal(ds)
 
@@ -416,3 +483,12 @@ def add_s_alpha_geometry_coords(ds, *, coordinates=None, grid=None):
         del ds["hthe"]
 
     return ds
+
+
+@register_geometry("fci")
+def add_fci_geometry_coords(ds, *, coordinates=None, grid=None):
+    assert coordinates is None, "Not implemented"
+    ds = _add_vars_from_grid(ds, grid, ["R", "Z"])
+    ds = ds.set_coords(("R", "Z"))
+    ds = _create_single_region(ds, periodic_y=True)
+    return ds
