@@ -24,13 +24,11 @@ _BOUT_PER_PROC_VARIABLES = [
     "wtime_per_rhs",
     "wtime_per_rhs_e",
     "wtime_per_rhs_i",
-    "hist_hi",
-    "tt",
     "PE_XIND",
     "PE_YIND",
     "MYPE",
 ]
-_BOUT_TIME_DEPENDENT_META_VARS = ["iteration"]
+_BOUT_TIME_DEPENDENT_META_VARS = ["iteration", "hist_hi", "tt"]
 
 
 # This code should run whenever any function from this module is imported
@@ -67,17 +65,33 @@ def open_boutdataset(
     keep_yboundaries=False,
     run_name=None,
     info=True,
+    is_restart=None,
     **kwargs,
 ):
     """
     Load a dataset from a set of BOUT output files, including the input options
-    file. Can also load from a grid file.
+    file. Can also load from a grid file or from restart files.
 
     Note that when reloading a Dataset that was saved by xBOUT, the state of the saved
     Dataset is restored, and the values of `keep_xboundaries`, `keep_yboundaries`, and
     `run_name` are ignored. `geometry` is treated specially, and can be passed when
     reloading a Dataset (along with `gridfilepath` if needed).
 
+    Troubleshooting
+    ---------------
+    Variable conflicts: sometimes, for example when loading data from multiple restarts,
+    some variables may have conflicts (e.g. a source term was changed between some of
+    the restarts, but the source term is saved as time-independent, without a
+    t-dimension). In this case one workaround is to pass a list of variable names to the
+    keyword argument `drop_vars` to ignore the variables with conflicts, e.g. if `"S1"`
+    and `"S2"` have conflicts
+    ```
+    ds = open_boutdataset("data*/boutdata.nc", drop_variables=["S1", "S2"])
+    ```
+    will open a Dataset which is missing `"S1"` and `"S2"`.\
+    [`drop_variables` is an argument of `xarray.open_dataset()` that is passed down
+    through `kwargs`.]
+
     Parameters
     ----------
     datapath : str or (list or tuple of xr.Dataset), optional
@@ -122,9 +136,15 @@ def open_boutdataset(
         Useful if you are going to open multiple simulations and compare the
         results.
     info : bool or "terse", optional
+    is_restart : bool, optional
+        Restart files require some special handling (e.g. working around variables that
+        are not present in restart files). By default, this special handling is enabled
+        if the files do not have a time dimension and `restart` is present in the file
+        name in `datapath`. This option can be set to True or False to explicitly enable
+        or disable the restart file handling.
     kwargs : optional
         Keyword arguments are passed down to `xarray.open_mfdataset`, which in
-        turn extra kwargs down to `xarray.open_dataset`.
+        turn passes extra kwargs down to `xarray.open_dataset`.
 
     Returns
     -------
@@ -136,8 +156,17 @@ def open_boutdataset(
 
     input_type = _check_dataset_type(datapath)
 
+    if is_restart is None:
+        is_restart = input_type == "restart"
+    elif is_restart is True:
+        input_type = "restart"
+
     if "reload" in input_type:
         if input_type == "reload":
+            if isinstance(datapath, Path):
+                # xr.open_mfdataset only accepts glob patterns as strings, not Path
+                # objects
+                datapath = str(datapath)
             ds = xr.open_mfdataset(
                 datapath,
                 chunks=chunks,
@@ -156,7 +185,10 @@ def open_boutdataset(
             sectionlength = len(section)
             for key in list(obj.attrs):
                 if key[:sectionlength] == section:
-                    result[key[sectionlength:]] = obj.attrs.pop(key)
+                    val = obj.attrs.pop(key)
+                    if isinstance(val, bytes):
+                        val = val.decode()
+                    result[key[sectionlength:]] = val
             return result
 
         def attrs_remove_section(obj, section):
@@ -171,6 +203,10 @@ def open_boutdataset(
 
         # Restore metadata from attrs
         metadata = attrs_to_dict(ds, "metadata")
+        if "is_restart" not in metadata:
+            # Loading data that was saved with a version of xbout from before
+            # "is_restart" was added, so need to add it to the metadata.
+            metadata["is_restart"] = int(is_restart)
         ds.attrs["metadata"] = metadata
         # Must do this for all variables and coordinates in dataset too
         for da in chain(ds.data_vars.values(), ds.coords.values()):
@@ -213,13 +249,14 @@ def open_boutdataset(
 
     # Determine if file is a grid file or data dump files
     remove_yboundaries = False
-    if "dump" in input_type:
+    if "dump" in input_type or "restart" in input_type:
         # Gather pointers to all numerical data from BOUT++ output files
         ds, remove_yboundaries = _auto_open_mfboutdataset(
             datapath=datapath,
             chunks=chunks,
             keep_xboundaries=keep_xboundaries,
             keep_yboundaries=keep_yboundaries,
+            is_restart=is_restart,
             **kwargs,
         )
     elif "grid" in input_type:
@@ -229,15 +266,26 @@ def open_boutdataset(
             chunks=chunks,
             keep_xboundaries=keep_xboundaries,
             keep_yboundaries=keep_yboundaries,
+            **kwargs,
         )
     else:
         raise ValueError(f"internal error: unexpected input_type={input_type}")
 
+    if not is_restart:
+        for var in _BOUT_TIME_DEPENDENT_META_VARS:
+            if var in ds:
+                # Assume different processors in x & y have same iteration etc.
+                latest_top_left = {dim: 0 for dim in ds[var].dims}
+                if "t" in ds[var].dims:
+                    latest_top_left["t"] = -1
+                ds[var] = ds[var].isel(latest_top_left).squeeze(drop=True)
+
     ds, metadata = _separate_metadata(ds)
     # Store as ints because netCDF doesn't support bools, so we can't save
     # bool attributes
     metadata["keep_xboundaries"] = int(keep_xboundaries)
     metadata["keep_yboundaries"] = int(keep_yboundaries)
+    metadata["is_restart"] = int(is_restart)
     ds = _set_attrs_on_all_vars(ds, "metadata", metadata)
 
     if remove_yboundaries:
@@ -245,14 +293,6 @@ def open_boutdataset(
         # grid file, as they will be removed from the full Dataset below
         keep_yboundaries = True
 
-    for var in _BOUT_TIME_DEPENDENT_META_VARS:
-        if var in ds:
-            # Assume different processors in x & y have same iteration etc.
-            latest_top_left = {dim: 0 for dim in ds[var].dims}
-            if "t" in ds[var].dims:
-                latest_top_left["t"] = -1
-            ds[var] = ds[var].isel(latest_top_left).squeeze(drop=True)
-
     ds = _add_options(ds, inputfilepath)
 
     if geometry is None:
@@ -327,9 +367,6 @@ def collect(
     info=True,
     prefix="BOUT.dmp",
 ):
-
-    from os.path import join
-
     """
 
     Extract the data pertaining to a specified variable in a BOUT++ data set
@@ -369,6 +406,7 @@ def collect(
     ds : numpy.ndarray
 
     """
+    from os.path import join
 
     datapath = join(path, prefix + "*.nc")
 
@@ -433,6 +471,8 @@ def _check_dataset_type(datapath):
         - only one file, and no time dimension
     (iii) produced by BOUT++
         - one or several files
+    (iv) restart files produced by BOUT++
+        - one or several files, no time dimension, filenames include `restart`
     """
 
     if not _is_path(datapath):
@@ -466,12 +506,18 @@ def _check_dataset_type(datapath):
     if "metadata:keep_yboundaries" in ds.attrs:
         # (i)
         return "reload"
-    elif len(filepaths) > 1 or "t" in ds.dims:
+    elif "t" in ds.dims:
         # (iii)
         return "dump"
-    else:
+    elif all(["restart" in Path(p).name for p in filepaths]):
+        # (iv)
+        return "restart"
+    elif len(filepaths) == 1:
         # (ii)
         return "grid"
+    else:
+        # fall back to opening as dump files
+        return "dump"
 
 
 def _auto_open_mfboutdataset(
@@ -480,17 +526,23 @@ def _auto_open_mfboutdataset(
     info=True,
     keep_xboundaries=False,
     keep_yboundaries=False,
+    is_restart=False,
     **kwargs,
 ):
     if chunks is None:
         chunks = {}
 
+    if is_restart:
+        data_vars = "minimal"
+    else:
+        data_vars = _BOUT_TIME_DEPENDENT_META_VARS
+
     if _is_path(datapath):
         filepaths, filetype = _expand_filepaths(datapath)
 
         # Open just one file to read processor splitting
         nxpe, nype, mxg, myg, mxsub, mysub, is_squashed_doublenull = _read_splitting(
-            filepaths[0], info
+            filepaths[0], info, keep_yboundaries
         )
 
         if is_squashed_doublenull:
@@ -510,6 +562,7 @@ def _auto_open_mfboutdataset(
             keep_boundaries={"x": keep_xboundaries, "y": keep_yboundaries},
             nxpe=nxpe,
             nype=nype,
+            is_restart=is_restart,
         )
 
         paths_grid, concat_dims = _arrange_for_concatenation(filepaths, nxpe, nype)
@@ -518,7 +571,7 @@ def _auto_open_mfboutdataset(
             paths_grid,
             concat_dim=concat_dims,
             combine="nested",
-            data_vars=_BOUT_TIME_DEPENDENT_META_VARS,
+            data_vars=data_vars,
             preprocess=_preprocess,
             engine=filetype,
             chunks=chunks,
@@ -556,6 +609,7 @@ def _auto_open_mfboutdataset(
             keep_boundaries={"x": keep_xboundaries, "y": keep_yboundaries},
             nxpe=nxpe,
             nype=nype,
+            is_restart=is_restart,
         )
 
         datapath = [_preprocess(x) for x in datapath]
@@ -565,15 +619,17 @@ def _auto_open_mfboutdataset(
         ds = xr.combine_nested(
             ds_grid,
             concat_dim=concat_dims,
-            data_vars=_BOUT_TIME_DEPENDENT_META_VARS,
+            data_vars=data_vars,
             join="exact",
             combine_attrs="no_conflicts",
         )
 
-    # Remove any duplicate time values from concatenation
-    _, unique_indices = unique(ds["t_array"], return_index=True)
+    if not is_restart:
+        # Remove any duplicate time values from concatenation
+        _, unique_indices = unique(ds["t_array"], return_index=True)
+        ds = ds.isel(t=unique_indices)
 
-    return ds.isel(t=unique_indices), remove_yboundaries
+    return ds, remove_yboundaries
 
 
 def _expand_filepaths(datapath):
@@ -619,7 +675,7 @@ def _expand_wildcards(path):
     return natsorted(filepaths, key=lambda filepath: str(filepath))
 
 
-def _read_splitting(filepath, info=True):
+def _read_splitting(filepath, info, keep_yboundaries):
     ds = xr.open_dataset(str(filepath))
 
     # Account for case of no parallelisation, when nxpe etc won't be in dataset
@@ -683,6 +739,23 @@ def _read_splitting(filepath, info=True):
                 nxpe = 1
                 nype = 1
                 is_squashed_doublenull = (ds["jyseps2_1"] != ds["jyseps1_2"]).values
+            elif ny_file == ny + 2 * myg:
+                # Older squashed file from double-null grid but containing only lower
+                # target boundary cells.
+                if keep_yboundaries:
+                    raise ValueError(
+                        "Cannot keep y-boundary points: squashed file is missing upper "
+                        "target boundary points."
+                    )
+                has_yboundaries = not (ny_file == ny)
+                if not has_yboundaries:
+                    myg = 0
+
+                nxpe = 1
+                nype = 1
+                # For this case, do not need the special handling enabled by
+                # is_squashed_doublenull=True, as keeping y-boundaries is not allowed
+                is_squashed_doublenull = False
 
     # Avoid trying to open this file twice
     ds.close()
@@ -732,7 +805,7 @@ def _arrange_for_concatenation(filepaths, nxpe=1, nype=1):
     return paths_grid, concat_dims
 
 
-def _trim(ds, *, guards, keep_boundaries, nxpe, nype):
+def _trim(ds, *, guards, keep_boundaries, nxpe, nype, is_restart):
     """
     Trims all guard (and optionally boundary) cells off a single dataset read from a
     single BOUT dump file, to prepare for concatenation.
@@ -750,6 +823,8 @@ def _trim(ds, *, guards, keep_boundaries, nxpe, nype):
         Number of processors in x direction
     nype : int
         Number of processors in y direction
+    is_restart : bool
+        Is data being loaded from restart files?
     """
 
     if any(keep_boundaries.values()):
@@ -774,7 +849,9 @@ def _trim(ds, *, guards, keep_boundaries, nxpe, nype):
         ):
             trimmed_ds = trimmed_ds.drop_vars(name)
 
-    return trimmed_ds.drop_vars(_BOUT_PER_PROC_VARIABLES, errors="ignore")
+    to_drop = _BOUT_PER_PROC_VARIABLES
+
+    return trimmed_ds.drop_vars(to_drop, errors="ignore")
 
 
 def _infer_contains_boundaries(ds, nxpe, nype):
@@ -842,7 +919,7 @@ def _get_limit(side, dim, keep_boundaries, boundaries, guards):
     return limit
 
 
-def _open_grid(datapath, chunks, keep_xboundaries, keep_yboundaries, mxg=2):
+def _open_grid(datapath, chunks, keep_xboundaries, keep_yboundaries, mxg=2, **kwargs):
     """
     Opens a single grid file. Implements slightly different logic for
     boundaries to deal with different conventions in a BOUT grid file.
@@ -860,7 +937,9 @@ def _open_grid(datapath, chunks, keep_xboundaries, keep_yboundaries, mxg=2):
 
     if _is_path(datapath):
         gridfilepath = Path(datapath)
-        grid = xr.open_dataset(gridfilepath, engine=_check_filetype(gridfilepath))
+        grid = xr.open_dataset(
+            gridfilepath, engine=_check_filetype(gridfilepath), **kwargs
+        )
     else:
         grid = datapath
 
@@ -876,17 +955,28 @@ def _open_grid(datapath, chunks, keep_xboundaries, keep_yboundaries, mxg=2):
         )
         grid = grid.drop_dims(unrecognised_dims)
 
-    if not keep_xboundaries:
+    if keep_xboundaries:
+        # Set MXG so that it is picked up in metadata - needed for applying geometry,
+        # etc.
+        grid["MXG"] = mxg
+    else:
         xboundaries = mxg
         if xboundaries > 0:
             grid = grid.isel(x=slice(xboundaries, -xboundaries, None))
-    if not keep_yboundaries:
-        try:
-            yboundaries = int(grid["y_boundary_guards"])
-        except KeyError:
-            # y_boundary_guards variable not in grid file - older grid files
-            # never had y-boundary cells
-            yboundaries = 0
+        # Set MXG so that it is picked up in metadata - needed for applying geometry,
+        # etc.
+        grid["MXG"] = 0
+    try:
+        yboundaries = int(grid["y_boundary_guards"])
+    except KeyError:
+        # y_boundary_guards variable not in grid file - older grid files
+        # never had y-boundary cells
+        yboundaries = 0
+    if keep_yboundaries:
+        # Set MYG so that it is picked up in metadata - needed for applying geometry,
+        # etc.
+        grid["MYG"] = yboundaries
+    else:
         if yboundaries > 0:
             # Remove y-boundary cells from first divertor target
             grid = grid.isel(y=slice(yboundaries, -yboundaries, None))
@@ -903,6 +993,9 @@ def _open_grid(datapath, chunks, keep_xboundaries, keep_yboundaries, mxg=2):
                     compat="identical",
                     join="exact",
                 )
+        # Set MYG so that it is picked up in metadata - needed for applying geometry,
+        # etc.
+        grid["MYG"] = 0
 
     if "z" in grid_chunks and "z" not in grid.dims:
         del grid_chunks["z"]
