@@ -81,7 +81,7 @@ def _normalise_time_coord(time_values):
     tmax = time_values.max()
     if tmax < 1.0e-2 or tmax > 1.0e6:
         scale_pow = int(np.floor(np.log10(tmax)))
-        scale_factor = 10 ** scale_pow
+        scale_factor = 10**scale_pow
         time_values = time_values / scale_factor
         suffix = f"e{scale_pow}"
     else:
@@ -221,7 +221,7 @@ def animate_poloidal(
         extend = "neither"
 
     # create colorbar
-    norm = _create_norm(logscale, kwargs.get("norm", None), vmin, vmax)
+    norm = _create_norm(logscale, kwargs.pop("norm", None), vmin, vmax)
     sm = plt.cm.ScalarMappable(norm=norm, cmap=cmap)
     sm.set_array([])
     cmap = sm.get_cmap()
@@ -262,6 +262,7 @@ def animate_poloidal(
                     da_region.values,
                     ax=ax,
                     cmap=cmap,
+                    norm=norm,
                     **kwargs,
                 )
             )
