@@ -116,13 +116,25 @@ class Region:
             # particular regions, so do not need to be consistent between different
             # regions (e.g. core and PFR), so we are not forced to use just the index
             # value here.
-            dx = ds["dx"].isel({self.ycoord: ylower_ind})
+            # Define ref_yind so that we avoid using values from the corner cells, which
+            # might be NaN.
+            if self.connection_lower_y is None and ds.metadata["keep_yboundaries"] == 1:
+                ref_yind = ylower_ind + ds.metadata["MYG"]
+            else:
+                ref_yind = ylower_ind
+            dx = ds["dx"].isel({self.ycoord: ref_yind})
             dx_cumsum = dx.cumsum()
             self.xinner = dx_cumsum[xinner_ind] - dx[xinner_ind]
             self.xouter = dx_cumsum[xouter_ind - 1] + dx[xouter_ind - 1]
 
             # dy is constant in the x-direction, so convert to a 1d array
-            dy = ds["dy"].isel(**{self.xcoord: self.xinner_ind})
+            # Define ref_xind so that we avoid using values from the corner cells, which
+            # might be NaN.
+            if self.connection_inner_x is None and ds.metadata["keep_xboundaries"] == 1:
+                ref_xind = xinner_ind + ds.metadata["MXG"]
+            else:
+                ref_xind = xinner_ind
+            dy = ds["dy"].isel(**{self.xcoord: ref_xind})
             dy_cumsum = dy.cumsum()
             self.ylower = dy_cumsum[ylower_ind] - dy[ylower_ind]
             self.yupper = dy_cumsum[yupper_ind - 1]
@@ -1212,6 +1224,37 @@ def _create_regions_toroidal(ds):
     return ds
 
 
+def _create_single_region(ds, periodic_y=True):
+    nx = ds.metadata["nx"]
+    ny = ds.metadata["ny"]
+
+    mxg = ds.metadata["MXG"]
+    myg = ds.metadata["MYG"]
+    # keep_yboundaries is 1 if there are y-boundaries and 0 if there are not
+    ybndry = ds.metadata["keep_yboundaries"] * myg
+
+    connection = "all" if periodic_y else None
+
+    regions = {
+        "all": Region(
+            name="all",
+            ds=ds,
+            xouter_ind=nx,
+            xinner_ind=0,
+            ylower_ind=0,
+            yupper_ind=ny,
+            connection_lower_y=connection,
+            connection_upper_y=connection,
+        )
+    }
+
+    _check_connections(regions)
+
+    ds = _set_attrs_on_all_vars(ds, "regions", regions)
+
+    return ds
+
+
 def _concat_inner_guards(da, da_global, mxg):
     """
     Concatenate inner x-guard cells to da, which is in a single region, getting the guard
@@ -1221,11 +1264,11 @@ def _concat_inner_guards(da, da_global, mxg):
     if mxg <= 0:
         return da
 
-    if len(da.regions) > 1:
+    if len(da.bout._regions) > 1:
         raise ValueError("da passed should have only one region")
-    region = list(da.regions.values())[0]
+    region = list(da.bout._regions.values())[0]
 
-    if region.connection_inner_x not in da_global.regions:
+    if region.connection_inner_x not in da_global.bout._regions:
         # No connection, or plotting restricted set of regions not including this
         # connection
         return da
@@ -1236,9 +1279,9 @@ def _concat_inner_guards(da, da_global, mxg):
     ycoord = da_global.metadata["bout_ydim"]
 
     da_inner = da_global.bout.from_region(region.connection_inner_x, with_guards=0)
-    if len(da_inner.regions) > 1:
+    if len(da_inner.bout._regions) > 1:
         raise ValueError("da_inner should have only one region")
-    region_inner = list(da_inner.regions.values())[0]
+    region_inner = list(da_inner.bout._regions.values())[0]
 
     if (
         myg_da > 0
@@ -1276,7 +1319,7 @@ def _concat_inner_guards(da, da_global, mxg):
         da_inner_lower = da_inner_lower.isel(
             **{xcoord: slice(-mxg, None), ycoord: slice(-myg_da, None)}
         )
-        save_regions = da_inner.regions
+        save_regions = da_inner.bout._regions
         da_inner = xr.concat((da_inner_lower, da_inner), ycoord, join="exact")
         # xr.concat takes attributes from the first variable, but we need da_inner's
         # regions
@@ -1314,7 +1357,7 @@ def _concat_inner_guards(da, da_global, mxg):
         da_inner[xcoord].data[...] = new_xcoord.data
         da_inner[ycoord].data[...] = new_ycoord.data
 
-    save_regions = da.regions
+    save_regions = da.bout._regions
     da = xr.concat((da_inner, da), xcoord, join="exact")
     # xr.concat takes attributes from the first variable (for xarray>=0.15.0, keeps attrs
     # that are the same in all objects for xarray<0.15.0)
@@ -1332,11 +1375,11 @@ def _concat_outer_guards(da, da_global, mxg):
     if mxg <= 0:
         return da
 
-    if len(da.regions) > 1:
+    if len(da.bout._regions) > 1:
         raise ValueError("da passed should have only one region")
-    region = list(da.regions.values())[0]
+    region = list(da.bout._regions.values())[0]
 
-    if region.connection_outer_x not in da_global.regions:
+    if region.connection_outer_x not in da_global.bout._regions:
         # No connection, or plotting restricted set of regions not including this
         # connection
         return da
@@ -1347,9 +1390,9 @@ def _concat_outer_guards(da, da_global, mxg):
     ycoord = da_global.metadata["bout_ydim"]
 
     da_outer = da_global.bout.from_region(region.connection_outer_x, with_guards=0)
-    if len(da_outer.regions) > 1:
+    if len(da_outer.bout._regions) > 1:
         raise ValueError("da_outer should have only one region")
-    region_outer = list(da_outer.regions.values())[0]
+    region_outer = list(da_outer.bout._regions.values())[0]
 
     if (
         myg_da > 0
@@ -1387,7 +1430,7 @@ def _concat_outer_guards(da, da_global, mxg):
         da_outer_lower = da_outer_lower.isel(
             **{xcoord: slice(-mxg, None), ycoord: slice(-myg_da, None)}
         )
-        save_regions = da_outer.regions
+        save_regions = da_outer.bout._regions
         da_outer = xr.concat((da_outer_lower, da_outer), ycoord, join="exact")
         # xr.concat takes attributes from the first variable, but we need da_outer's
         # regions
@@ -1425,7 +1468,7 @@ def _concat_outer_guards(da, da_global, mxg):
         da_outer[xcoord].data[...] = new_xcoord.data
         da_outer[ycoord].data[...] = new_ycoord.data
 
-    save_regions = da.regions
+    save_regions = da.bout._regions
     da = xr.concat((da, da_outer), xcoord, join="exact")
     # xarray<0.15.0 only keeps attrs that are the same on all variables passed to concat
     da.attrs["regions"] = save_regions
@@ -1442,11 +1485,11 @@ def _concat_lower_guards(da, da_global, mxg, myg):
     if myg <= 0:
         return da
 
-    if len(da.regions) > 1:
+    if len(da.bout._regions) > 1:
         raise ValueError("da passed should have only one region")
-    region = list(da.regions.values())[0]
+    region = list(da.bout._regions.values())[0]
 
-    if region.connection_lower_y not in da_global.regions:
+    if region.connection_lower_y not in da_global.bout._regions:
         # No connection, or plotting restricted set of regions not including this
         # connection
         return da
@@ -1525,7 +1568,23 @@ def _concat_lower_guards(da, da_global, mxg, myg):
         da_lower[xcoord].data[...] = new_xcoord.data
         da_lower[ycoord].data[...] = new_ycoord.data
 
-    save_regions = da.regions
+    if "poloidal_distance" in da.coords and myg > 0:
+        # Special handling for core regions to deal with branch cut
+        if "core" in region.name:
+            # import pdb; pdb.set_trace()
+            # Try to detect whether there is branch cut at lower boundary: if there is
+            # poloidal_distance_ylow should be zero at the boundary of this region
+            poloidal_distance_bottom = da["poloidal_distance_ylow"].isel({ycoord: 0})
+            if all(abs(poloidal_distance_bottom) < 1.0e-16):
+                # Offset so that the poloidal_distance in da_lower is continuous from
+                # the poloidal_distance in this region.
+                # Expect there to be y-boundary cells in the Dataset, this will probably
+                # fail if there are not.
+                total_poloidal_distance = da["total_poloidal_distance"]
+                da_lower["poloidal_distance"] -= total_poloidal_distance
+                da_lower["poloidal_distance_ylow"] -= total_poloidal_distance
+
+    save_regions = da.bout._regions
     da = xr.concat((da_lower, da), ycoord, join="exact")
     # xr.concat takes attributes from the first variable (for xarray>=0.15.0, keeps attrs
     # that are the same in all objects for xarray<0.15.0)
@@ -1543,11 +1602,11 @@ def _concat_upper_guards(da, da_global, mxg, myg):
     if myg <= 0:
         return da
 
-    if len(da.regions) > 1:
+    if len(da.bout._regions) > 1:
         raise ValueError("da passed should have only one region")
-    region = list(da.regions.values())[0]
+    region = list(da.bout._regions.values())[0]
 
-    if region.connection_upper_y not in da_global.regions:
+    if region.connection_upper_y not in da_global.bout._regions:
         # No connection, or plotting restricted set of regions not including this
         # connection
         return da
@@ -1625,7 +1684,25 @@ def _concat_upper_guards(da, da_global, mxg, myg):
         da_upper[xcoord].data[...] = new_xcoord.data
         da_upper[ycoord].data[...] = new_ycoord.data
 
-    save_regions = da.regions
+    if "poloidal_distance" in da.coords and myg > 0:
+        # Special handling for core regions to deal with branch cut
+        if "core" in region.name:
+            # import pdb; pdb.set_trace()
+            # Try to detect whether there is branch cut at upper boundary: if there is
+            # poloidal_distance_ylow should be zero at the boundary of da_upper
+            poloidal_distance_bottom = da_upper["poloidal_distance_ylow"].isel(
+                {ycoord: 0}
+            )
+            if all(abs(poloidal_distance_bottom) < 1.0e-16):
+                # Offset so that the poloidal_distance in da_upper is continuous from
+                # the poloidal_distance in this region.
+                # Expect there to be y-boundary cells in the Dataset, this will probably
+                # fail if there are not.
+                total_poloidal_distance = da["total_poloidal_distance"]
+                da_upper["poloidal_distance"] += total_poloidal_distance
+                da_upper["poloidal_distance_ylow"] += total_poloidal_distance
+
+    save_regions = da.bout._regions
     da = xr.concat((da, da_upper), ycoord, join="exact")
     # xarray<0.15.0 only keeps attrs that are the same on all variables passed to concat
     da.attrs["regions"] = save_regions
@@ -1637,7 +1714,7 @@ def _from_region(ds_or_da, name, with_guards):
     # ensure we do not modify the input
     ds_or_da = ds_or_da.copy(deep=True)
 
-    region = ds_or_da.regions[name]
+    region = ds_or_da.bout._regions[name]
     xcoord = ds_or_da.metadata["bout_xdim"]
     ycoord = ds_or_da.metadata["bout_ydim"]
 
@@ -1669,7 +1746,7 @@ def _from_region(ds_or_da, name, with_guards):
 
     # If the result (which only has a single region) is passed to from_region a
     # second time, don't want to slice anything.
-    single_region = list(result.regions.values())[0]
+    single_region = list(result.bout._regions.values())[0]
     single_region.xinner_ind = None
     single_region.xouter_ind = None
     single_region.ylower_ind = None
