@@ -64,11 +64,12 @@ def _separate_metadata(ds):
         var
         for var in variables
         if not any(dim in ["t", "x", "y", "z"] for dim in ds[var].dims)
-        and not var in exclude
+        and var not in exclude
     ]
 
     # Save metadata as a dictionary
     metadata_vals = [ds[var].values.item() for var in scalar_vars]
+    metadata_vals = [x.decode() if isinstance(x, bytes) else x for x in metadata_vals]
     metadata = dict(zip(scalar_vars, metadata_vals))
 
     # Add default values for dimensions to metadata. These may be modified later by
@@ -195,6 +196,32 @@ def _1d_coord_from_spacing(spacing, dim, ds=None, *, origin_at=None):
     return xr.Variable(dim, coord_values)
 
 
+def _add_cartesian_coordinates(ds):
+    # Add Cartesian X and Y coordinates if they do not exist already
+    # Works on either BoutDataset or BoutDataArray
+
+    if ds.geometry == "toroidal":
+        R = ds["R"]
+        Z = ds["Z"]
+        zeta = ds[ds.metadata["bout_zdim"]]
+        if "X_cartesian" not in ds.coords:
+            X = R * np.cos(zeta)
+            ds = ds.assign_coords(X_cartesian=X)
+        if "Y_cartesian" not in ds.coords:
+            Y = R * np.sin(zeta)
+            ds = ds.assign_coords(Y_cartesian=Y)
+        if "Z_cartesian" not in ds.coords:
+            zcoord = ds.metadata["bout_zdim"]
+            nz = len(ds[zcoord])
+            ds = ds.assign_coords(Z_cartesian=Z.expand_dims({zcoord: nz}, axis=-1))
+    else:
+        raise ValueError(
+            f"Adding Cartesian coordinates to geometry={ds.geometry} is not supported"
+        )
+
+    return ds
+
+
 def _check_new_nxpe(ds, nxpe):
     # Check nxpe is valid
 
@@ -315,6 +342,12 @@ def _pad_y_boundaries(ds):
 
 
 def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, overwrite):
+    # Rename dimension coordinates to BOUT++ standard names
+    ds = ds.copy()
+    for d in ["t", "x", "y", "z"]:
+        key = f"bout_{d}dim"
+        if key in ds.metadata and ds.metadata[key] in ds.dims:
+            ds = ds.rename({ds.metadata[key]: d})
 
     _check_new_nxpe(ds, nxpe)
     _check_new_nype(ds, nype)
@@ -338,37 +371,28 @@ def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, over
 
     ny_inner = ds.metadata["ny_inner"]
 
-    tcoord = ds.metadata.get("bout_tdim", "t")
-    xcoord = ds.metadata.get("bout_xdim", "x")
-    ycoord = ds.metadata.get("bout_ydim", "y")
-
-    # These variables need to be saved to restart files in addition to evolving ones
-    restart_metadata_vars = [
-        "zperiod",
-        "MZSUB",
-        "MXG",
-        "MYG",
-        "MZG",
-        "nx",
-        "ny",
-        "nz",
-        "MZ",
-        "NZPE",
-        "ixseps1",
-        "ixseps2",
-        "jyseps1_1",
-        "jyseps2_1",
-        "jyseps1_2",
-        "jyseps2_2",
-        "ny_inner",
-        "ZMAX",
-        "ZMIN",
-        "BOUT_VERSION",
+    # These metadata variables are created by xBOUT, so should not be saved to restart
+    # files
+    restart_exclude_metadata_vars = [
+        "bout_tdim",
+        "bout_xdim",
+        "bout_ydim",
+        "bout_zdim",
+        "fine_interpolation_factor",
+        "is_restart",
+        "keep_xboundaries",
+        "keep_yboundaries",
     ]
 
     if variables is None:
-        # If variables to be saved were not specified, add all time-evolving variables
-        variables = [v for v in ds if tcoord in ds[v].dims]
+        if "t" in ds.dims:
+            # If variables to be saved were not specified, add all time-evolving
+            # variables
+            variables = [v for v in ds if "t" in ds[v].dims]
+        else:
+            # No time dimension in Dataset, so cannot distinguish time-evolving
+            # variables: just include all variables
+            variables = [v for v in ds]
 
     # Add extra variables always needed
     for v in [
@@ -388,8 +412,9 @@ def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, over
         "g_13",
         "g_23",
         "J",
+        "zShift",
     ]:
-        if v not in variables:
+        if v not in variables and v in ds:
             variables.append(v)
 
     # number of points in the domain on each processor, not including guard or boundary
@@ -399,16 +424,39 @@ def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, over
 
     # hist_hi represents the number of iterations before the restart. Attempt to
     # reconstruct here
-    iteration = ds.metadata.get("iteration", -1)
-    nt = ds.sizes[tcoord]
-    hist_hi = iteration - (nt - tind)
-    if hist_hi < 0:
-        hist_hi = -1
+    final_hist_hi = ds.metadata.get("hist_hi", -1)
+    if "t" in ds.dims:
+        nt = ds.sizes["t"]
+        if tind < 0:
+            absolute_tind = tind + nt
+            if absolute_tind < 0:
+                absolute_tind = 0
+        elif tind >= nt:
+            absolute_tind = nt - 1
+        else:
+            absolute_tind = tind
+        hist_hi = final_hist_hi - (nt - 1 - absolute_tind)
+        if hist_hi < 0:
+            hist_hi = -1
+    else:
+        # Either input is a set of restart files, in which case we should just use
+        # `hist_hi` if it exists, or the user has already selected a single time-point,
+        # and the best guess we have left is the existing `hist_hi` (although it might
+        # not be right).
+        hist_hi = final_hist_hi
 
     has_second_divertor = ds.metadata["jyseps2_1"] != ds.metadata["jyseps1_2"]
 
-    # select desired time-index for the restart files
-    ds = ds.isel({tcoord: tind}).persist()
+    if "t" in ds.dims:
+        # select desired time-index for the restart files
+        ds = ds.isel({"t": tind}).persist()
+        tt = ds["t"].values.flatten()[0]
+    else:
+        if "tt" in ds.metadata:
+            # If loaded from restart files, "tt" should be a scalar in metadata
+            tt = ds.metadata["tt"]
+        else:
+            tt = 0.0
 
     ds = _pad_x_boundaries(ds)
     ds = _pad_y_boundaries(ds)
@@ -423,7 +471,7 @@ def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, over
             else:
                 yslice = slice(yproc * mysub, (yproc + 1) * mysub + 2 * myg)
 
-            ds_slice = ds.isel({xcoord: xslice, ycoord: yslice})
+            ds_slice = ds.isel({"x": xslice, "y": yslice})
 
             restart_ds = xr.Dataset()
             for v in variables:
@@ -433,16 +481,25 @@ def _split_into_restarts(ds, variables, savepath, nxpe, nype, tind, prefix, over
                 data_variable.attrs = {}
 
                 restart_ds[v] = data_variable
-            for v in restart_metadata_vars:
-                restart_ds[v] = ds.metadata[v]
+            for v in ds.metadata:
+                if v not in restart_exclude_metadata_vars:
+                    restart_ds[v] = ds.metadata[v]
+
+            # These variables need to be altered, because they depend on the number of
+            # files and/or the rank of this file.
             restart_ds["MXSUB"] = mxsub
             restart_ds["MYSUB"] = mysub
             restart_ds["NXPE"] = nxpe
             restart_ds["NYPE"] = nype
+            restart_ds["PE_XIND"] = xproc
+            restart_ds["PE_YIND"] = yproc
             restart_ds["hist_hi"] = hist_hi
+            restart_ds["PE_XIND"] = xproc
+            restart_ds["PE_YIND"] = yproc
+            restart_ds["MYPE"] = yproc * nxpe + xproc
 
             # tt is the simulation time where the restart happens
-            restart_ds["tt"] = ds[tcoord].values.flatten()[0]
+            restart_ds["tt"] = tt
 
             restart_datasets.append(restart_ds)
 
@@ -456,7 +513,7 @@ _bounding_surface_checks = {}
 
 
 def _check_upper_y(ds_region, boundary_points, xbndry, ybndry, Rcoord, Zcoord):
-    region = list(ds_region.regions.values())[0]
+    region = list(ds_region.bout._regions.values())[0]
     xcoord = ds_region.metadata["bout_xdim"]
     ycoord = ds_region.metadata["bout_ydim"]
 
@@ -489,7 +546,7 @@ _bounding_surface_checks["upper_y"] = _check_upper_y
 
 
 def _check_inner_x(ds_region, boundary_points, xbndry, ybndry, Rcoord, Zcoord):
-    region = list(ds_region.regions.values())[0]
+    region = list(ds_region.bout._regions.values())[0]
     xcoord = ds_region.metadata["bout_xdim"]
     ycoord = ds_region.metadata["bout_ydim"]
 
@@ -522,7 +579,7 @@ _bounding_surface_checks["inner_x"] = _check_inner_x
 
 
 def _check_lower_y(ds_region, boundary_points, xbndry, ybndry, Rcoord, Zcoord):
-    region = list(ds_region.regions.values())[0]
+    region = list(ds_region.bout._regions.values())[0]
     xcoord = ds_region.metadata["bout_xdim"]
     ycoord = ds_region.metadata["bout_ydim"]
 
@@ -550,7 +607,7 @@ _bounding_surface_checks["lower_y"] = _check_lower_y
 
 
 def _check_outer_x(ds_region, boundary_points, xbndry, ybndry, Rcoord, Zcoord):
-    region = list(ds_region.regions.values())[0]
+    region = list(ds_region.bout._regions.values())[0]
     xcoord = ds_region.metadata["bout_xdim"]
     ycoord = ds_region.metadata["bout_ydim"]
 
@@ -597,7 +654,7 @@ def _follow_boundary(ds, start_region, start_direction, xbndry, ybndry, Rcoord,
         visited_regions.append(this_region)
 
         ds_region = ds.bout.from_region(this_region, with_guards=0)
-        region = ds.regions[this_region]
+        region = ds.bout._regions[this_region]
 
         # Get all boundary points from this region, and decide which region to go to next
         this_region = None
@@ -665,14 +722,14 @@ def _get_bounding_surfaces(ds, coords):
 
     # First find the outer boundary
     start_region = None
-    for name, region in ds.regions.items():
+    for name, region in ds.bout._regions.items():
         if region.connection_lower_y is None and region.connection_outer_x is None:
             start_region = name
             break
     if start_region is None:
         # No y-boundary region found, presumably is core-only simulation. Start on any
         # region with an outer-x boundary
-        for name, region in ds.regions.items():
+        for name, region in ds.bout._regions.items():
             if region.connection_outer_x is None:
                 start_region = name
     if start_region is None:
@@ -682,7 +739,7 @@ def _get_bounding_surfaces(ds, coords):
 
     # First region has outer-x boundary, but we only visit start_region once, so need to
     # add all boundaries in it the first time.
-    region = ds.regions[start_region]
+    region = ds.bout._regions[start_region]
     if region.connection_upper_y is None:
         start_direction = "inner_x"
     elif region.connection_inner_x is None and region.connection_lower_y is None:
@@ -705,7 +762,7 @@ def _get_bounding_surfaces(ds, coords):
 
     # Look for an inner boundary
     ############################
-    remaining_regions = set(ds.regions) - set(checked_regions)
+    remaining_regions = set(ds.bout._regions) - set(checked_regions)
     start_region = None
     if not remaining_regions:
         # Check for separate inner-x boundary on any of the already visited regions.
@@ -715,9 +772,9 @@ def _get_bounding_surfaces(ds, coords):
         # a separate inner boundary
         for r in checked_regions:
             if (
-                ds.regions[r].connection_inner_x is None
-                and ds.regions[r].connection_lower_y is not None
-                and ds.regions[r].connection_upper_y is not None
+                ds.bout._regions[r].connection_inner_x is None
+                and ds.bout._regions[r].connection_lower_y is not None
+                and ds.bout._regions[r].connection_upper_y is not None
                 and checked_regions.count(r) < 2
             ):
                 start_region = r
@@ -725,9 +782,9 @@ def _get_bounding_surfaces(ds, coords):
     else:
         for r in remaining_regions:
             if (
-                ds.regions[r].connection_inner_x is None
-                and ds.regions[r].connection_lower_y is not None
-                and ds.regions[r].connection_upper_y is not None
+                ds.bout._regions[r].connection_inner_x is None
+                and ds.bout._regions[r].connection_lower_y is not None
+                and ds.bout._regions[r].connection_upper_y is not None
             ):
                 start_region = r
                 break
@@ -744,11 +801,11 @@ def _get_bounding_surfaces(ds, coords):
         boundaries.append(boundary)
         checked_regions += more_checked_regions
 
-        remaining_regions = set(ds.regions) - set(checked_regions)
+        remaining_regions = set(ds.bout._regions) - set(checked_regions)
 
     # If there are any remaining regions, they should not have any boundaries
     for r in remaining_regions:
-        region = ds.regions[r]
+        region = ds.bout._regions[r]
         if (
             region.connection_lower_y is None
             or region.connection_outer_x is None
